(* %Tiny *)(* %Rem  NiceList for Atari-ST *)
(* 'NiceList' a program by Harrie F.A. de Leeuw,  Sept-1986 *)
(* Version 1.0 *)

MODULE NiceList;


FROM SYSTEM        IMPORT ADR;
FROM OClock        IMPORT TimeRecord, ReadTime;
FROM TextIO        IMPORT WRite, WRiteString, WRiteLn, WRiteCard,
                          REad, REadString, REadLn;
FROM Streams       IMPORT OpenStream, CloseStream, StreamKinds, Stream, EOS;
FROM String        IMPORT Strings, Assign, Concat, Pos;
FROM ASCII         IMPORT FF;
FROM GEMDOS        IMPORT SFirst, SNext, SetDTA, SetDrv, SetPath;
FROM AESForms      IMPORT FormAlert;

FROM NLSetUp       IMPORT SetDouble, SetEnlarge, SetEmphasize, SetCondense,
                          SetUnderline, SetSubscript, SetSuperscript,
                          ResetDouble, ResetEnlarge, ResetEmphasize,
                          ResetCondense, ResetUnderline, ResetSubscript,
                          ResetSuperscript, SetSpacing, DotsPerLine,
                          NrOfLinesPerPage, FormLength;
FROM NLGEM         IMPORT Directory, Drive, FileName, SelectedButton,
                          SelectedButtonType, InitProgram, MenuForm,
                          EndProgram;
FROM NLOutWindow   IMPORT OpenOutWindow, WriteOutWindow, CloseOutWindow;



(* %F10 *)
TYPE PrintStates= (Enl,Emp,Con,Dou,Und,Sup,Sub);
     PrStateSet=  SET OF PrintStates;
     DTABuffer=   RECORD
                    Fill: ARRAY[0..19] OF CHAR;
                    Attr: CARDINAL;
                    Time: CARDINAL;
                    Date: CARDINAL;
                    Size: LONGCARD;
                    Name: ARRAY[0..14] OF CHAR
                  END;



(* %F32 *)
VAR TempSet,
    StatesSet:       PrStateSet;
    DirSlot,
    LinePoint,
    SearchPoint,
    InPoint,
    LinesAdd,
    TempLinesAdd,
    LinesPrinted,
    PageNr,
    NewPoint:        CARDINAL;
    Line:            ARRAY[0..131] OF CHAR;
    OutLine,
    InString,
    FilNam:          Strings;
    SearchString:    ARRAY[0..19] OF CHAR;
    Ch:              CHAR;
    NoneSelected,
    Bool,
    First,
    PrintHeader:     BOOLEAN;
    Mon:             ARRAY[0..39] OF CHAR;
    InText,
    OutText:         Stream;
    AlertBut,
    Result,
    StreamState:     INTEGER;
    DrvMap:          LONGCARD;
    DTA:             POINTER TO Strings;
    DTABuf:          DTABuffer;
    ResetRoot:       ARRAY[0..1] OF CHAR;



(* %F8 *)
PROCEDURE ControlError;
BEGIN
   WriteOutWindow('Error in listing control character: ');
   Assign(OutLine,'       ');
   Concat(OutLine,Line,OutLine);
   WriteOutWindow(OutLine);
   WRiteLn(OutText)
END ControlError;


(* %F32 *)
PROCEDURE ReadNr(VAR Nr: CARDINAL);
BEGIN
  LinePoint:=LinePoint+1;
  CASE Line[LinePoint] OF
    '0': Nr:=0  |
    '1': Nr:=1  |
    '2': Nr:=2  |
    '3': Nr:=3  |
    '4': Nr:=4  |
    '5': Nr:=5  |
    '6': Nr:=6  |
    '7': Nr:=7  |
    '8': Nr:=8  |
    '9': Nr:=9
  ELSE
    Nr:=1000
  END;
  LinePoint:=LinePoint+1;
  CASE Line[LinePoint] OF
    '0': Nr:=Nr*10      |
    '1': Nr:=Nr*10+1    |
    '2': Nr:=Nr*10+2    |
    '3': Nr:=Nr*10+3    |
    '4': Nr:=Nr*10+4    |
    '5': Nr:=Nr*10+5    |
    '6': Nr:=Nr*10+6    |
    '7': Nr:=Nr*10+7    |
    '8': Nr:=Nr*10+8    |
    '9': Nr:=Nr*10+9
  ELSE
  END
END ReadNr;


(* %F11 *)
PROCEDURE WROut(VAR S:Strings);
VAR Len, Poin: CARDINAL;
BEGIN
  Len:=ORD(S[0]);
  Poin:=1;
  WHILE Len>0 DO
    WRite(OutText,S[Poin]);
    Poin:=Poin+1;
    Len:=Len-1
  END
END WROut;


(* %F5 *)
PROCEDURE Enlarge;
BEGIN
  WROut(SetEnlarge);
  INCL(StatesSet,Enl)
END Enlarge;


(* %F5 *)
PROCEDURE Condense;
BEGIN
  WROut(SetCondense);
  INCL(StatesSet,Con)
END Condense;


(* %F5 *)
PROCEDURE Emphasize;
BEGIN
  WROut(SetEmphasize);
  INCL(StatesSet,Emp)
END Emphasize;


(* %F5 *)
PROCEDURE Double;
BEGIN
  WROut(SetDouble);
  INCL(StatesSet,Dou)
END Double;


(* %F5 *)
PROCEDURE Underline;
BEGIN
  WROut(SetUnderline);
  INCL(StatesSet,Und)
END Underline;


(* %F4 *)
PROCEDURE Bell;
BEGIN
  WRite(OutText,CHR(7))
END Bell;


(* %F5 *)
PROCEDURE SuperScript;
BEGIN
  WROut(SetSuperscript);
  INCL(StatesSet,Sup)
END SuperScript;


(* %F5 *)
PROCEDURE SubScript;
BEGIN
  WROut(SetSubscript);
  INCL(StatesSet,Sub)
END SubScript;


(* %F7 *)
PROCEDURE SetLineSpacing;
BEGIN
  IF SetSpacing[0]#0C THEN
    WROut(SetSpacing);
    WRite(OutText,CHR(LinesAdd))
  END
END SetLineSpacing;


(* %F11 *)
PROCEDURE LineSpacing;
VAR LinSpa: CARDINAL;
BEGIN
  ReadNr(LinSpa);
  IF LinSpa>85 THEN
    ControlError
  ELSE
    LinesAdd:=LinSpa;
    SetLineSpacing
  END
END LineSpacing;


(* %F7 *)
PROCEDURE Tiny;
BEGIN
  Condense;
  SubScript;
  LinesAdd:=6;
  SetLineSpacing
END Tiny;


(* %F19 *)
PROCEDURE Normal;
BEGIN
  WROut(ResetSubscript);
  WROut(ResetSuperscript);
  WROut(ResetUnderline);
  WROut(ResetDouble);
  WROut(ResetEmphasize);
  WROut(ResetCondense);
  WROut(ResetEnlarge);
  LinesAdd:=DotsPerLine;
  SetLineSpacing;
  EXCL(StatesSet,Enl);
  EXCL(StatesSet,Emp);
  EXCL(StatesSet,Con);
  EXCL(StatesSet,Dou);
  EXCL(StatesSet,Und);
  EXCL(StatesSet,Sup);
  EXCL(StatesSet,Sub)
END Normal;


(* %F6 *)
PROCEDURE Remark;
BEGIN
  WriteOutWindow(Line);
  Line:='(* %R *)';
  LinePoint:=4
END Remark;


(* %F11 *)
PROCEDURE SupSubLineSpace;
BEGIN
  LinePoint:=LinePoint+1;
  CASE Line[LinePoint] OF
    'U': SuperScript    |
    'B': SubScript      |       
    'P': LineSpacing
  ELSE
    ControlError
  END
END SupSubLineSpace;


(* %F11 *)
PROCEDURE Header;
BEGIN
  LinePoint:=LinePoint+1;
  IF Line[LinePoint]='+' THEN
    PrintHeader:=TRUE
  ELSIF Line[LinePoint]='-' THEN
    PrintHeader:=FALSE
  ELSE
    ControlError
  END
END Header;


(* %F5 *)
PROCEDURE SaveStates;
BEGIN
  TempSet:=StatesSet;
  TempLinesAdd:=LinesAdd
END SaveStates;


(* %F13 *)
PROCEDURE RestoreStates;
BEGIN
  LinesAdd:=TempLinesAdd;
  StatesSet:=TempSet;
  IF Enl IN StatesSet THEN Enlarge END;
  IF Emp IN StatesSet THEN Emphasize END;
  IF Con IN StatesSet THEN Condense END;
  IF Dou IN StatesSet THEN Double END;
  IF Und IN StatesSet THEN Underline END;
  IF Sup IN StatesSet THEN SuperScript END;
  IF Sub IN StatesSet THEN SubScript END;
  SetLineSpacing
END RestoreStates;


(* %F7 *)
PROCEDURE NewPage;
BEGIN
  WRite(OutText,FF);
  LinesPrinted:=0;
  PageNr:=PageNr+1;
  PrintHeading
END NewPage;


(* %F7 *)
PROCEDURE TestFF;
BEGIN
  LinesPrinted:=LinesPrinted+LinesAdd;
  IF LinesPrinted>=FormLength THEN
    NewPage
  END
END TestFF;


(* %F22 *)
PROCEDURE PrintHeading;
VAR Step: CARDINAL;
    TimeRec: TimeRecord;
BEGIN
  IF PrintHeader THEN
    SaveStates;
    Normal;
    WRiteString(OutText,'FileName: ');
    Enlarge;
    Emphasize;
    Underline;
    WRiteString(OutText,FilNam);
    Normal;
    Step:=0;
    WHILE FilNam[Step]#0C DO
      Step:=Step+1
    END;
    Step:=Step*2;
    WHILE Step<34 DO
      WRite(OutText,' ');
      Step:=Step+1
    END;
(* %F31 *)
    ReadTime(TimeRec);
    WRiteString(OutText,'Date: ');
    Emphasize;
    WRiteCard(OutText,TimeRec.Day,2);
    WRite(OutText,'-');
    WRite(OutText,Mon[TimeRec.Month*3]);
    WRite(OutText,Mon[TimeRec.Month*3+1]);
    WRite(OutText,Mon[TimeRec.Month*3+2]);
    WRite(OutText,'-');
    WRiteCard(OutText,TimeRec.Year MOD 100,2);
    WRiteString(OutText,'  ');
    WRiteCard(OutText,TimeRec.Hour,2);
    WRite(OutText,':');
    IF TimeRec.Min<10 THEN
      WRite(OutText,'0')
    END;
    WRiteCard(OutText,TimeRec.Min,1);
    WRite(OutText,'h');
    Normal;
    WRiteString(OutText,'   Page: ');
    Emphasize;
    Underline;
    WRiteCard(OutText,PageNr,1);
    Normal;
    WRiteLn(OutText);
    TestFF;
    WRiteLn(OutText);
    TestFF;
    RestoreStates
  END
END PrintHeading;


(* %F8 *)
PROCEDURE FormFeed;
VAR MinLeft: CARDINAL;
BEGIN
  ReadNr(MinLeft);
  IF ((FormLength-LinesPrinted)<=(MinLeft*LinesAdd)) THEN
    NewPage
  END
END FormFeed;


(* %F12 *)
PROCEDURE Haakje;
VAR Where: CARDINAL;
    EndOk: BOOLEAN;
  PROCEDURE NoControl;
  BEGIN
    IF First THEN
      PrintHeading
    END;
    WRiteLn(OutText);
    TestFF;
    WRiteString(OutText,Line)
  END NoControl;
(* %F25 *)
BEGIN (* Haakje *)
  IF Line[1]='*' THEN
    LinePoint:=0;
    WHILE ((Line[LinePoint]#'%') & (Line[LinePoint]#0C)) DO
                                                     (* Skip spaces *)
      LinePoint:=LinePoint+1
    END;
    IF Line[LinePoint]='%' THEN
      LinePoint:=LinePoint+1;
      CASE Line[LinePoint] OF
        'F': FormFeed   |
        'E': Enlarge    |
        'C': Condense   |
        'M': Emphasize  |
        'D': Double     |
        'U': Underline  |
        'N': Normal     |
        'B': Bell       |
        'H': Header     |
        'R': Remark     |
        'T': Tiny       |
        'S': SupSubLineSpace
      ELSE
        ControlError
      END;
(* %F12 *)
      EndOk:=FALSE;
      IF Pos(Line,'*)',LinePoint,Where) THEN
        LinePoint:=Where;
        EndOk:=TRUE
      ELSE
        IF Pos(Line,'*/',LinePoint,Where) THEN
          LinePoint:=Where;
          EndOk:=TRUE
        ELSE
          ControlError  (* No end delimeter *)
        END
      END;
(* %F26 *)
      IF EndOk THEN
        LinePoint:=LinePoint+2;
        NewPoint:=0;
        WHILE Line[LinePoint]#0C DO
          Line[NewPoint]:=Line[LinePoint];
          LinePoint:=LinePoint+1;
          NewPoint:=NewPoint+1
        END;
        Line[NewPoint]:=0C;
        IF ((Line[0]='(') OR (Line[0]='/')) THEN
          Haakje
        ELSE
          IF First THEN
            PrintHeading
          END;
          WRiteString(OutText,Line)
        END
      END
    ELSE
      NoControl
    END
  ELSE
    NoControl
  END
END Haakje;


(* %F24 *)
PROCEDURE PrintFile(FileN: Strings);
BEGIN
  OpenStream(InText,FileN,READ,StreamState);
  IF StreamState#0 THEN
    AlertBut:=FormAlert(1,'[3][ |OpenStream failed!][Exit]');
    HALT
  END;
  NoneSelected:=FALSE;
  PageNr:=1;
  Assign(OutLine,'Printing: ');
  Concat(OutLine,FileN,OutLine);
  WriteOutWindow(OutLine);
  Assign(FilNam,FileN);
  PrintHeader:=TRUE;
  REadString(InText,Line);
  First:=TRUE;
  IF ((Line[0]='(') OR (Line[0]='/')) THEN
    Haakje
  ELSE
    PrintHeading;
    WRiteLn(OutText);
    TestFF;
    WRiteString(OutText,Line)
  END;
(* %F22 *)
  REadLn(InText);
  REadString(InText,Line);
  First:=FALSE;
  WHILE NOT(EOS(InText)) DO
    REadLn(InText);
    IF ((Line[0]='(') OR (Line[0]='/')) THEN
      Haakje
    ELSE
      WRiteLn(OutText);
      TestFF;
      WRiteString(OutText,Line)
    END;
    IF NOT(EOS(InText)) THEN
      REadString(InText,Line)
    END
  END;
  CloseStream(InText,StreamState);
  IF StreamState#0 THEN
    AlertBut:=FormAlert(1,'[3][ |CloseStream failed!][Exit]');
    HALT
  END
END PrintFile;


(* %F13 *)
PROCEDURE PrintFiles;
BEGIN
  SFirst(SearchString,0,Result);
  WHILE Result=0 DO
    LinesPrinted:=0;
    LinesAdd:=DotsPerLine;
    WriteOutWindow('  ');
    PrintFile(DTABuf.Name);
    WRite(OutText,FF);
    Normal;
    SNext(Result)
  END
END PrintFiles;



(* %F19 *)
BEGIN   (* NiceList *)
  InitProgram;    (* GEM initialisations *)
  Mon:='   JanFebMarAprMayJunJulAugSepOctNovDec';
  EXCL(StatesSet,Enl);
  EXCL(StatesSet,Emp);
  EXCL(StatesSet,Con);
  EXCL(StatesSet,Dou);
  EXCL(StatesSet,Und);
  EXCL(StatesSet,Sup);
  EXCL(StatesSet,Sub);
  DTA:=ADR(DTABuf);
  SetDTA(DTA);
  ResetRoot[0]:='\';
  ResetRoot[1]:=0C;
  OpenStream(OutText,'prn:',READWRITE,StreamState);
  IF StreamState#0 THEN
    AlertBut:=FormAlert(1,'[3][ |prn: can not be opened!][Exit]');
    HALT
  END;
(* %F35 *)
  REPEAT
    MenuForm;
    IF SelectedButton=Start THEN
      OpenOutWindow;
      NoneSelected:=TRUE;
      SetDrv(Drive,DrvMap);
      Bool:=SetPath(ResetRoot);
      Bool:=SetPath(Directory);
      IF ~Bool THEN
        AlertBut:=FormAlert(1,'[1][ |Directory not found!][Ok]')
      ELSE
        Assign(InString,FileName);
        InPoint:=0;
        WHILE InString[InPoint]#0C DO
          SearchPoint:=0;
          WHILE ((InString[InPoint]#0C) & (InString[InPoint]#',')) DO
            SearchString[SearchPoint]:=InString[InPoint];
            InPoint:=InPoint+1;
            SearchPoint:=SearchPoint+1
          END;
          SearchString[SearchPoint]:=0C;
          PrintFiles;
          IF InString[InPoint]=',' THEN
            InPoint:=InPoint+1
          END
        END
      END;
      IF NoneSelected THEN
        AlertBut:=FormAlert(1,'[1][ |No files selected!][Ok]')
      ELSE
        AlertBut:=FormAlert(1,"[1][ |That's all folks!][Ok]")
      END;
      CloseOutWindow
    END
  UNTIL SelectedButton=Quit;
(* %F7 *)
  CloseStream(OutText,StreamState);
  IF StreamState#0 THEN
    AlertBut:=FormAlert(1,'[3][ |Error closing prn: !][Exit]');
    HALT
  END;
  EndProgram    (* Close workstation *)
END NiceList.
